@@ -15,7 +15,7 @@ from unittest import mock
 from django.conf import settings
 from django.core import management, signals
 from django.core.cache import (
-    DEFAULT_CACHE_ALIAS, CacheKeyWarning, InvalidCacheKey, cache, caches,
+    DEFAULT_CACHE_ALIAS, CacheKeyWarning, cache, caches,
 )
 from django.core.cache.utils import make_template_fragment_key
 from django.db import close_old_connections, connection, connections
@@ -274,6 +274,11 @@ class BaseCacheTests:
         cache.set("key", "value")
         self.assertEqual(cache.get("key"), "value")
 
+    def test_default_used_when_none_is_set(self):
+        """If None is cached, get() returns it instead of the default."""
+        cache.set('key_default_none', None)
+        self.assertIsNone(cache.get('key_default_none', default='default'))
+
     def test_add(self):
         # A key can be added to a cache
         cache.add("addkey1", "value")
@@ -605,10 +610,10 @@ class BaseCacheTests:
 
     def _perform_invalid_key_test(self, key, expected_warning):
         """
-        All the builtin backends should warn (except memcached that should
-        error) on keys that would be refused by memcached. This encourages
-        portable caching code without making it too difficult to use production
-        backends with more liberal key rules. Refs #6447.
+        All the builtin backends (except memcached, see below) should warn on
+        keys that would be refused by memcached. This encourages portable
+        caching code without making it too difficult to use production backends
+        with more liberal key rules. Refs #6447.
         """
         # mimic custom ``make_key`` method being defined since the default will
         # never show the below warnings
@@ -619,9 +624,8 @@ class BaseCacheTests:
         cache.key_func = func
 
         try:
-            with self.assertWarns(CacheKeyWarning) as cm:
+            with self.assertWarnsMessage(CacheKeyWarning, expected_warning):
                 cache.set(key, 'value')
-            self.assertEqual(str(cm.warning), expected_warning)
         finally:
             cache.key_func = old_func
 
@@ -1252,15 +1256,24 @@ class BaseMemcachedTests(BaseCacheTests):
                 with self.settings(CACHES={'default': params}):
                     self.assertEqual(cache._servers, ['server1.tld', 'server2:11211'])
 
-    def _perform_invalid_key_test(self, key, expected_warning):
+    def test_invalid_key_characters(self):
         """
-        Whilst other backends merely warn, memcached should raise for an
-        invalid key.
+        On memcached, we don't introduce a duplicate key validation
+        step (for speed reasons), we just let the memcached API
+        library raise its own exception on bad keys. Refs #6447.
+
+        In order to be memcached-API-library agnostic, we only assert
+        that a generic exception of some kind is raised.
         """
-        msg = expected_warning.replace(key, cache.make_key(key))
-        with self.assertRaises(InvalidCacheKey) as cm:
-            cache.set(key, 'value')
-        self.assertEqual(str(cm.exception), msg)
+        # memcached does not allow whitespace or control characters in keys
+        # when using the ascii protocol.
+        with self.assertRaises(Exception):
+            cache.set('key with spaces', 'value')
+
+    def test_invalid_key_length(self):
+        # memcached limits key length to 250
+        with self.assertRaises(Exception):
+            cache.set('a' * 251, 'value')
 
     def test_default_never_expiring_timeout(self):
         # Regression test for #22845
@@ -1357,6 +1370,14 @@ class MemcachedCacheTests(BaseMemcachedTests, TestCase):
     def test_memcached_options(self):
         self.assertEqual(cache._cache.server_max_value_length, 9999)
 
+    def test_default_used_when_none_is_set(self):
+        """
+        python-memcached doesn't support default in get() so this test
+        overrides the one in BaseCacheTests.
+        """
+        cache.set('key_default_none', None)
+        self.assertEqual(cache.get('key_default_none', default='default'), 'default')
+
 
 @unittest.skipUnless(PyLibMCCache_params, "PyLibMCCache backend not configured")
 @override_settings(CACHES=caches_setting_for_tests(
@@ -1369,6 +1390,15 @@ class PyLibMCCacheTests(BaseMemcachedTests, TestCase):
     # libmemcached manages its own connections.
     should_disconnect_on_close = False
 
+    # By default, pylibmc/libmemcached don't verify keys client-side and so
+    # this test triggers a server-side bug that causes later tests to fail
+    # (#19914). The `verify_keys` behavior option could be set to True (which
+    # would avoid triggering the server-side bug), however this test would
+    # still fail due to https://github.com/lericson/pylibmc/issues/219.
+    @unittest.skip("triggers a memcached-server bug, causing subsequent tests to fail")
+    def test_invalid_key_characters(self):
+        pass
+
     @override_settings(CACHES=caches_setting_for_tests(
         base=PyLibMCCache_params,
         exclude=memcached_excluded_caches,
@@ -1422,7 +1452,7 @@ class FileBasedCacheTests(BaseCacheTests, TestCase):
     def test_creates_cache_dir_if_nonexistent(self):
         os.rmdir(self.dirname)
         cache.set('foo', 'bar')
-        os.path.exists(self.dirname)
+        self.assertTrue(os.path.exists(self.dirname))
 
     def test_get_ignores_enoent(self):
         cache.set('foo', 'bar')
@@ -1431,8 +1461,8 @@ class FileBasedCacheTests(BaseCacheTests, TestCase):
         self.assertEqual(cache.get('foo', 'baz'), 'baz')
 
     def test_get_does_not_ignore_non_filenotfound_exceptions(self):
-        with mock.patch('builtins.open', side_effect=IOError):
-            with self.assertRaises(IOError):
+        with mock.patch('builtins.open', side_effect=OSError):
+            with self.assertRaises(OSError):
                 cache.get('foo')
 
     def test_empty_cache_file_considered_expired(self):
@@ -1595,6 +1625,8 @@ class CacheUtils(SimpleTestCase):
             (None, ('Accept-Encoding', 'COOKIE'), 'Accept-Encoding, COOKIE'),
             ('Cookie,     Accept-Encoding', ('Accept-Encoding', 'cookie'), 'Cookie, Accept-Encoding'),
             ('Cookie    ,     Accept-Encoding', ('Accept-Encoding', 'cookie'), 'Cookie, Accept-Encoding'),
+            ('*', ('Accept-Language', 'Cookie'), '*'),
+            ('Accept-Language, Cookie', ('*',), '*'),
         )
         for initial_vary, newheaders, resulting_vary in headers:
             with self.subTest(initial_vary=initial_vary, newheaders=newheaders):
