@@ -1,53 +1,32 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
-import datetime
-
 from django.contrib import admin
 from django.contrib.admin.sites import AdminSite
 from django.contrib.auth.models import User
 from django.contrib.contenttypes.admin import GenericTabularInline
-from django.contrib.contenttypes.forms import generic_inlineformset_factory
 from django.contrib.contenttypes.models import ContentType
-from django.core.urlresolvers import reverse
 from django.forms.formsets import DEFAULT_MAX_NUM
 from django.forms.models import ModelForm
 from django.test import (
     RequestFactory, SimpleTestCase, TestCase, override_settings,
 )
+from django.urls import reverse
 
 from .admin import MediaInline, MediaPermanentInline, site as admin_site
 from .models import Category, Episode, EpisodePermanent, Media, PhoneNumber
 
 
-class TestDataMixin(object):
+class TestDataMixin:
 
     @classmethod
     def setUpTestData(cls):
-        # password = "secret"
-        User.objects.create(
-            pk=100, username='super', first_name='Super', last_name='User', email='super@example.com',
-            password='sha1$995a3$6011485ea3834267d719b4c801409b8b1ddd0158', is_active=True, is_superuser=True,
-            is_staff=True, last_login=datetime.datetime(2007, 5, 30, 13, 20, 10),
-            date_joined=datetime.datetime(2007, 5, 30, 13, 20, 10)
-        )
+        cls.superuser = User.objects.create_superuser(username='super', password='secret', email='super@example.com')
 
 
-# Set DEBUG to True to ensure {% include %} will raise exceptions.
-# That is how inlines are rendered and #9498 will bubble up if it is an issue.
-@override_settings(
-    DEBUG=True,
-    PASSWORD_HASHERS=['django.contrib.auth.hashers.SHA1PasswordHasher'],
-    ROOT_URLCONF="generic_inline_admin.urls",
-)
+@override_settings(ROOT_URLCONF='generic_inline_admin.urls')
 class GenericAdminViewTest(TestDataMixin, TestCase):
 
     def setUp(self):
-        self.client.login(username='super', password='secret')
+        self.client.force_login(self.superuser)
 
-        # Can't load content via a fixture (since the GenericForeignKey
-        # relies on content type IDs, which will vary depending on what
-        # other tests have been run), thus we do it here.
         e = Episode.objects.create(name='This Week in Django')
         self.episode_pk = e.pk
         m = Media(content_object=e, url='http://example.com/podcast.mp3')
@@ -109,119 +88,13 @@ class GenericAdminViewTest(TestDataMixin, TestCase):
         response = self.client.post(url, post_data)
         self.assertEqual(response.status_code, 302)  # redirect somewhere
 
-    def test_generic_inline_formset(self):
-        EpisodeMediaFormSet = generic_inlineformset_factory(
-            Media,
-            can_delete=False,
-            exclude=['description', 'keywords'],
-            extra=3,
-        )
-        e = Episode.objects.get(name='This Week in Django')
 
-        # Works with no queryset
-        formset = EpisodeMediaFormSet(instance=e)
-        self.assertEqual(len(formset.forms), 5)
-        self.assertHTMLEqual(
-            formset.forms[0].as_p(),
-            '<p><label for="id_generic_inline_admin-media-content_type-object_id-0-url">'
-            'Url:</label> <input id="id_generic_inline_admin-media-content_type-object_id-0-url" '
-            'type="url" name="generic_inline_admin-media-content_type-object_id-0-url" '
-            'value="http://example.com/podcast.mp3" maxlength="200" />'
-            '<input type="hidden" name="generic_inline_admin-media-content_type-object_id-0-id" '
-            'value="%s" id="id_generic_inline_admin-media-content_type-object_id-0-id" /></p>'
-            % self.mp3_media_pk
-        )
-        self.assertHTMLEqual(
-            formset.forms[1].as_p(),
-            '<p><label for="id_generic_inline_admin-media-content_type-object_id-1-url">'
-            'Url:</label> <input id="id_generic_inline_admin-media-content_type-object_id-1-url" '
-            'type="url" name="generic_inline_admin-media-content_type-object_id-1-url" '
-            'value="http://example.com/logo.png" maxlength="200" />'
-            '<input type="hidden" name="generic_inline_admin-media-content_type-object_id-1-id" '
-            'value="%s" id="id_generic_inline_admin-media-content_type-object_id-1-id" /></p>'
-            % self.png_media_pk
-        )
-        self.assertHTMLEqual(
-            formset.forms[2].as_p(),
-            '<p><label for="id_generic_inline_admin-media-content_type-object_id-2-url">Url:</label>'
-            '<input id="id_generic_inline_admin-media-content_type-object_id-2-url" type="url" '
-            'name="generic_inline_admin-media-content_type-object_id-2-url" maxlength="200" />'
-            '<input type="hidden" name="generic_inline_admin-media-content_type-object_id-2-id" '
-            'id="id_generic_inline_admin-media-content_type-object_id-2-id" /></p>'
-        )
-
-        # A queryset can be used to alter display ordering
-        formset = EpisodeMediaFormSet(instance=e, queryset=Media.objects.order_by('url'))
-        self.assertEqual(len(formset.forms), 5)
-        self.assertHTMLEqual(
-            formset.forms[0].as_p(),
-            '<p><label for="id_generic_inline_admin-media-content_type-object_id-0-url">Url:</label>'
-            '<input id="id_generic_inline_admin-media-content_type-object_id-0-url" type="url" '
-            'name="generic_inline_admin-media-content_type-object_id-0-url"'
-            'value="http://example.com/logo.png" maxlength="200" />'
-            '<input type="hidden" name="generic_inline_admin-media-content_type-object_id-0-id" '
-            'value="%s" id="id_generic_inline_admin-media-content_type-object_id-0-id" /></p>'
-            % self.png_media_pk
-        )
-        self.assertHTMLEqual(
-            formset.forms[1].as_p(),
-            '<p><label for="id_generic_inline_admin-media-content_type-object_id-1-url">Url:</label>'
-            '<input id="id_generic_inline_admin-media-content_type-object_id-1-url" type="url" '
-            'name="generic_inline_admin-media-content_type-object_id-1-url" '
-            'value="http://example.com/podcast.mp3" maxlength="200" />'
-            '<input type="hidden" name="generic_inline_admin-media-content_type-object_id-1-id" '
-            'value="%s" id="id_generic_inline_admin-media-content_type-object_id-1-id" /></p>'
-            % self.mp3_media_pk
-        )
-        self.assertHTMLEqual(
-            formset.forms[2].as_p(),
-            '<p><label for="id_generic_inline_admin-media-content_type-object_id-2-url">'
-            'Url:</label> <input id="id_generic_inline_admin-media-content_type-object_id-2-url" '
-            'type="url" name="generic_inline_admin-media-content_type-object_id-2-url" maxlength="200" />'
-            '<input type="hidden" name="generic_inline_admin-media-content_type-object_id-2-id" '
-            'id="id_generic_inline_admin-media-content_type-object_id-2-id" /></p>'
-        )
-
-        # Works with a queryset that omits items
-        formset = EpisodeMediaFormSet(instance=e, queryset=Media.objects.filter(url__endswith=".png"))
-        self.assertEqual(len(formset.forms), 4)
-        self.assertHTMLEqual(
-            formset.forms[0].as_p(),
-            '<p><label for="id_generic_inline_admin-media-content_type-object_id-0-url">Url:</label>'
-            ' <input id="id_generic_inline_admin-media-content_type-object_id-0-url" type="url" '
-            'name="generic_inline_admin-media-content_type-object_id-0-url" '
-            'value="http://example.com/logo.png" maxlength="200" />'
-            '<input type="hidden" name="generic_inline_admin-media-content_type-object_id-0-id" '
-            'value="%s" id="id_generic_inline_admin-media-content_type-object_id-0-id" /></p>'
-            % self.png_media_pk
-        )
-        self.assertHTMLEqual(
-            formset.forms[1].as_p(),
-            '<p><label for="id_generic_inline_admin-media-content_type-object_id-1-url">'
-            'Url:</label> <input id="id_generic_inline_admin-media-content_type-object_id-1-url" '
-            'type="url" name="generic_inline_admin-media-content_type-object_id-1-url" maxlength="200" />'
-            '<input type="hidden" name="generic_inline_admin-media-content_type-object_id-1-id" '
-            'id="id_generic_inline_admin-media-content_type-object_id-1-id" /></p>'
-        )
-
-    def test_generic_inline_formset_factory(self):
-        # Regression test for #10522.
-        inline_formset = generic_inlineformset_factory(Media,
-            exclude=('url',))
-
-        # Regression test for #12340.
-        e = Episode.objects.get(name='This Week in Django')
-        formset = inline_formset(instance=e)
-        self.assertTrue(formset.get_queryset().ordered)
-
-
-@override_settings(PASSWORD_HASHERS=['django.contrib.auth.hashers.SHA1PasswordHasher'],
-                   ROOT_URLCONF="generic_inline_admin.urls")
+@override_settings(ROOT_URLCONF='generic_inline_admin.urls')
 class GenericInlineAdminParametersTest(TestDataMixin, TestCase):
+    factory = RequestFactory()
 
     def setUp(self):
-        self.client.login(username='super', password='secret')
-        self.factory = RequestFactory()
+        self.client.force_login(self.superuser)
 
     def _create_object(self, model):
         """
@@ -263,7 +136,7 @@ class GenericInlineAdminParametersTest(TestDataMixin, TestCase):
         self.assertEqual(formset.total_form_count(), 1)
         self.assertEqual(formset.initial_form_count(), 1)
 
-    def testMaxNumParam(self):
+    def test_max_num_param(self):
         """
         With extra=5 and max_num=2, there should be only 2 forms.
         """
@@ -361,12 +234,11 @@ class GenericInlineAdminParametersTest(TestDataMixin, TestCase):
         self.assertEqual(formset.max_num, 2)
 
 
-@override_settings(PASSWORD_HASHERS=['django.contrib.auth.hashers.SHA1PasswordHasher'],
-                   ROOT_URLCONF="generic_inline_admin.urls")
+@override_settings(ROOT_URLCONF='generic_inline_admin.urls')
 class GenericInlineAdminWithUniqueTogetherTest(TestDataMixin, TestCase):
 
     def setUp(self):
-        self.client.login(username='super', password='secret')
+        self.client.force_login(self.superuser)
 
     def test_add(self):
         category_id = Category.objects.create(name='male').pk
@@ -397,7 +269,7 @@ class GenericInlineAdminWithUniqueTogetherTest(TestDataMixin, TestCase):
         self.assertContains(response, 'Are you sure you want to delete')
 
 
-@override_settings(ROOT_URLCONF="generic_inline_admin.urls")
+@override_settings(ROOT_URLCONF='generic_inline_admin.urls')
 class NoInlineDeletionTest(SimpleTestCase):
 
     def test_no_deletion(self):
@@ -407,19 +279,20 @@ class NoInlineDeletionTest(SimpleTestCase):
         self.assertFalse(formset.can_delete)
 
 
-class MockRequest(object):
+class MockRequest:
     pass
 
 
-class MockSuperUser(object):
+class MockSuperUser:
     def has_perm(self, perm):
         return True
 
+
 request = MockRequest()
 request.user = MockSuperUser()
 
 
-@override_settings(ROOT_URLCONF="generic_inline_admin.urls")
+@override_settings(ROOT_URLCONF='generic_inline_admin.urls')
 class GenericInlineModelAdminTest(SimpleTestCase):
 
     def setUp(self):
@@ -431,16 +304,16 @@ class GenericInlineModelAdminTest(SimpleTestCase):
         # Create a formset with default arguments
         formset = media_inline.get_formset(request)
         self.assertEqual(formset.max_num, DEFAULT_MAX_NUM)
-        self.assertEqual(formset.can_order, False)
+        self.assertIs(formset.can_order, False)
 
         # Create a formset with custom keyword arguments
         formset = media_inline.get_formset(request, max_num=100, can_order=True)
         self.assertEqual(formset.max_num, 100)
-        self.assertEqual(formset.can_order, True)
+        self.assertIs(formset.can_order, True)
 
     def test_custom_form_meta_exclude_with_readonly(self):
         """
-        Ensure that the custom ModelForm's `Meta.exclude` is respected when
+        The custom ModelForm's `Meta.exclude` is respected when
         used in conjunction with `GenericInlineModelAdmin.readonly_fields`
         and when no `ModelAdmin.exclude` is defined.
         """
@@ -467,7 +340,7 @@ class GenericInlineModelAdminTest(SimpleTestCase):
 
     def test_custom_form_meta_exclude(self):
         """
-        Ensure that the custom ModelForm's `Meta.exclude` is respected by
+        The custom ModelForm's `Meta.exclude` is respected by
         `GenericInlineModelAdmin.get_formset`, and overridden if
         `ModelAdmin.exclude` or `GenericInlineModelAdmin.exclude` are defined.
         Refs #15907.
@@ -512,7 +385,7 @@ class GenericInlineModelAdminTest(SimpleTestCase):
             ['description', 'keywords', 'id', 'DELETE'])
 
     def test_get_fieldsets(self):
-        # Test that get_fieldsets is called when figuring out form fields.
+        # get_fieldsets is called when figuring out form fields.
         # Refs #18681.
         class MediaForm(ModelForm):
             class Meta:
@@ -533,7 +406,7 @@ class GenericInlineModelAdminTest(SimpleTestCase):
 
     def test_get_formsets_with_inlines_returns_tuples(self):
         """
-        Ensure that get_formsets_with_inlines() returns the correct tuples.
+        get_formsets_with_inlines() returns the correct tuples.
         """
         class MediaForm(ModelForm):
             class Meta:
@@ -556,3 +429,29 @@ class GenericInlineModelAdminTest(SimpleTestCase):
         inlines = ma.get_inline_instances(request)
         for (formset, inline), other_inline in zip(ma.get_formsets_with_inlines(request), inlines):
             self.assertIsInstance(formset, other_inline.get_formset(request).__class__)
+
+    def test_get_inline_instances_override_get_inlines(self):
+        class MediaInline(GenericTabularInline):
+            model = Media
+
+        class AlternateInline(GenericTabularInline):
+            model = Media
+
+        class EpisodeAdmin(admin.ModelAdmin):
+            inlines = (AlternateInline, MediaInline)
+
+            def get_inlines(self, request, obj):
+                if hasattr(request, 'name'):
+                    if request.name == 'alternate':
+                        return self.inlines[:1]
+                    elif request.name == 'media':
+                        return self.inlines[1:2]
+                return []
+
+        ma = EpisodeAdmin(Episode, self.site)
+        self.assertEqual(ma.get_inlines(request, None), [])
+        self.assertEqual(ma.get_inline_instances(request), [])
+        for name, inline_class in (('alternate', AlternateInline), ('media', MediaInline)):
+            request.name = name
+            self.assertEqual(ma.get_inlines(request, None), (inline_class,)),
+            self.assertEqual(type(ma.get_inline_instances(request)[0]), inline_class)
